************** Parms **************
clear *
set more off
global path ~/replication_eer
global bsreps 999
parallel setclusters 10, force // here we assume 10 cpus

************** Progs **************

* net install fhetprob, from(http://www.richard-bluhm.com/stata/)
* ssc install parallel, replace
* mata mata mlib index

************* Prelims *************

cd $path
use data/fullpanel0_march2013.dta

drop ryear
gen ryear = round(year)

* restrict sample to best coverage
drop if ryear<1981 | ryear>2010

* get PCE per capita from PWT71

merge m:1 ccode ryear using ./data/pwt71_w_consumption2.dta
drop if _merge==2
xtset ccode_cov ryear
drop _merge

* generate vars
xtset ccode_cov ryear

gen ln_mean = ln(mean)
gen ln_gini = ln(gini)
gen ln_rcp = ln(rcp)
gen rcp12 = rcp/12

label var ln_mean "$ \ln \bar{y}_{it} $"
label var ln_gini "$ \ln G_{it} $"
label var h_200d "$ H_{it} $"
label var ln_rcp "$ \ln PCE^P_{it} $"

encode regWB, gen(region) //region dummies

**** clean 

egen anymiss = rowmiss(h_200d ln_mean ln_gini)
drop if anymiss

*** final data starts here

************* Table 1 *************
sum h_200d h_125d gini mean rcp12 ibn.stype
xi, noomit: sutex h_200d h_125d gini mean rcp12 i.stype,  file(./tables/Table_1.tex) ///
	replace minmax digits(3) 
***********************************

************ Table A-2 ************
bys regWB : sum h_200d gini mean 

bys regWB : sutex h_200d gini mean, file(./tables/Table_A_2.tex)  ///
	replace minmax  digits(3)
***********************************
	
************* Table 2 *************

* prep unbalanced panel stuff
bys ccode_cov (ryear): gen Ti = _N 
tab Ti if ~anymiss, gen(Ti_)

tab stype if ~anymiss, gen(S_)

bys ccode_cov (ryear): egen ln_mean_bar = mean(ln_mean)
bys ccode_cov (ryear): egen ln_gini_bar = mean(ln_gini) 
bys ccode_cov (ryear): egen Sb_2 = mean(S_2) 
bys ccode_cov (ryear): egen Sb_3 = mean(S_3) 
bys ccode_cov (ryear): egen Sb_4 = mean(S_4) 
forv i = 1981(1)2010 {
	gen y_`i' =  (ryear==`i') 
	bys ccode_cov (ryear): egen yb_`i' = mean(y_`i') 
}


* GLM, column 1
glm h_200d  ln_mean ln_mean_bar ln_gini ln_gini_bar y_1981-yb_2009 S_2-S_4 Sb_* if Ti>1, ///
	fam(bin) link(probit) vce(cluster ccode_cov)
	
estimate store glm
margins, eydx(ln_mean ln_gini) nose	

predictnl  double scale=normalden(xb()) if e(sample) 
sum scale
local scale = r(mean)

predict double yhat if e(sample)
corr yhat h_200d
local r2 = r(rho)^2
gen se = (h_200d - yhat)^2 
sum se, mean
local rmse = sqrt(r(mean))
 
outreg2 using ./tables/Table_2.tex, replace keep(ln_mean ln_gini) noobs ///
	addtext("CRE - Corr. Rand. Effects", Yes, ///
	"Survey type dummies", Yes, ///
	"Time dummies", Yes, ///
	"Panel size dummies", No, ///
	"Panel size $ \times $ CRE", No, ///
	"Variance equation", No ) /// 
	addstat(Scale Factor, `scale', ///
	$ N \times \bar{T} $, e(N), ///
	$ N $, e(N_clust), $ pseudo-R^2 $, `r2', $ \ln \mathcal{L} $ , e(ll), $ \sqrt{MSE} $, `rmse') ///
	nocons label eq(auto) ct("$ H_{it} $") noaster nonotes dec(3) 
	
margins, dydx(ln_mean ln_gini)  post

outreg2 using ./tables/Table_2.tex, append keep(ln_mean ln_gini) noobs ///
	nocons label eq(auto) ct("APEs") noaster nonotes dec(3)  
	
* flexible unbalanced, column2
fhetprob h_200d  ln_mean ln_gini ib(last).Ti##c.ln_mean_bar ib(last).Ti##c.ln_gini_bar ///
	y_1981-yb_2009 S_2-S_4 Sb_*  if Ti>1, het(ib(last).Ti) vce(cluster ccode_cov)	
	
estimate store fhet
margins, eydx(ln_mean ln_gini) nose	

cap drop scale
predictnl double scale=normalden(xb(#1)/exp(xb(#2)))*exp(-xb(#2)) if e(sample) 
sum scale
local scale = r(mean)

cap drop yhat 
cap drop se
predict double yhat if e(sample)
corr yhat h_200d
local r2 = r(rho)^2
gen se = (h_200d - yhat)^2 
sum se, mean
local rmse = sqrt(r(mean))
dis `rmse'

outreg2 using ./tables/Table_2.tex, append eqkeep(h200_d) keep(ln_mean ln_gini) noobs ///
	addtext("CRE - Corr. Rand. Effects", Yes, ///
	"Survey type dummies", Yes, ///
	"Time dummies", Yes, ///
	"Panel size dummies", Yes, ///
	"Panel size $ \times $ CRE", Yes, ///
	"Variance equation", Yes ) /// 
	addstat(Scale Factor, `scale', ///
	$ N \times \bar{T} $, e(N), ///
	$ N $, e(N_clust), $ pseudo-R^2 $, `r2', $ \ln \mathcal{L} $ , e(ll), $ \sqrt{MSE} $, `rmse') ///
	nocons label eq(auto) ct("$ H_{it} $")   noaster dec(3)  
	
margins, dydx(ln_mean ln_gini)  post

outreg2 using ./tables/Table_2.tex, append keep(ln_mean ln_gini) noobs ///
	nocons label eqkeep(APEs) ct("APEs") noaster nonotes dec(3)  


* accounting for measurement error, restrict sample to IV, recreate unbal panel stuff

cap drop anymiss ln_mean_bar ln_gini_bar y_* yb_* Ti Ti_* S_* Sb_*
egen anymiss = rowmiss(h_200d ln_mean ln_gini ln_rcp)
drop if anymiss

bys ccode_cov (ryear): gen Ti = _N 
tab Ti, gen(T_) 

tab stype, gen(S_)

bys ccode_cov (ryear): egen ln_mean_bar = mean(ln_mean)
bys ccode_cov (ryear): egen ln_gini_bar = mean(ln_gini)
bys ccode_cov (ryear): egen ln_rcp_bar = mean(ln_rcp)

bys ccode_cov (ryear): egen Sb_2 = mean(S_2)
bys ccode_cov (ryear): egen Sb_3 = mean(S_3)
bys ccode_cov (ryear): egen Sb_4 = mean(S_4)

forv i = 1981(1)2010 {
	gen y_`i' =  (ryear==`i')
	bys ccode_cov (ryear): egen yb_`i' = mean(y_`i')
}

* control function: 1st step
reg ln_mean ln_rcp ln_gini ///
	ib(last).Ti##c.ln_rcp_bar ib(last).Ti##c.ln_gini_bar ///
	y_1981-yb_2009 S_2-S_4 Sb_* if Ti>1, vce(cluster ccode_cov)
testparm ln_rcp 

predict v2hat if e(sample), resid
label var v2hat "$ \hat{\nu}_{it} $"


* control function: 2nd step, not conflating first stage u with endo

bys ccode_cov (ryear): egen ln_mean_nbar = mean(ln_mean) if e(sample)
fhetprob h_200d  ln_mean ln_gini v2hat ib(last).Ti##c.ln_mean_nbar ///
	ib(last).Ti##c.ln_gini_bar ib(last).Ti##c.ln_rcp_bar ///
	y_1981-yb_2009 S_2-S_4 Sb_* if Ti>1, het(ib(last).Ti) vce(cluster ccode_cov)
estimates store varfhet
test v2hat

margins, eydx(ln_mean ln_gini) nose


cap drop scale
predictnl double scale=normalden(xb(#1)/exp(xb(#2)))*exp(-xb(#2)) if e(sample) 
sum scale
local scale = r(mean)	

cap drop yhat se
predict double yhat if e(sample)
corr yhat h_200d
local r2 = r(rho)^2
gen se = (h_200d - yhat)^2 
sum se, mean
local rmse = sqrt(r(mean))
local aic = e(aic)
local ll = e(ll)

cap program drop twostep_boot

program twostep_boot, eclass
version 12.0
tempname bb v2h meanbar conv
	* control function: 1st step
	reg ln_mean ln_rcp ln_gini ib(last).Ti##c.ln_rcp_bar ib(last).Ti##c.ln_gini_bar ///
		y_1981-yb_2009 S_2-S_4 Sb_* if Ti>1
	predict `v2h', resid

	bys newid (ryear): egen `meanbar' = mean(ln_mean) if e(sample)
	
	* control function: 2nd step, set iter 100 to unjam too long bs iterations
	fhetprob h_200d  ln_mean ln_gini `v2h' ib(last).Ti##c.`meanbar' ib(last).Ti##c.ln_rcp_bar ib(last).Ti##c.ln_gini_bar ///
		y_1981-yb_2009  S_2-S_4 Sb_* if Ti>1, het(ib(last).Ti) iter(100)
	
	scalar `conv' = e(converged)

	mat `bb' = e(b)
	mat `bb' = `bb'[1,1..3]	
	
	margins, dydx(ln_mean ln_gini) nose
	mat `bb' = `bb',r(b)
	mat coln `bb' =  h_200d:ln_mean h_200d:ln_gini h_200d:v2hat APEs:ln_mean APEs:ln_gini
	ereturn post `bb', depn(h_200d)
	ereturn scalar converged =`conv'

end

preserve

xtset, clear

drop if Ti==1 

parallel bs, reps(`=$bsreps') ///
 seeds(10203 20301 30201 20103 40506 50604 60504 50406 70809 80907) ///
 cluster(ccode_cov) idcluster(newid) nowarn reject(e(converged)==0): twostep_boot
xtset ccode_cov ryear

* something goes wrong with outreg2 here, have to manually delete the duplicate coefs
* the SEs in the APE column are not correct, but can be found at the bottom of the previous column	
outreg2 using ./tables/Table_2.tex, append eqkeep(h_200d) keep(ln_mean ln_gini v2hat)  noobs ///
	addtext("CRE - Corr. Rand. Effects", Yes, ///
	"Survey type dummies", Yes, ///
	"Time dummies", Yes, ///
	"Panel size dummies", Yes, ///
	"Panel size $ \times $ CRE", Yes, ///
	"Variance equation", Yes ) /// 
	addstat(Scale Factor, `scale', ///
	$ N \times \bar{T} $, e(N), ///
	$ N $, e(N_clust), $ pseudo-R^2 $, `r2', $ \ln \mathcal{L} $ , `ll', $ \sqrt{MSE} $, `rmse') ///
	nocons label ct("$ H_{it} $") noaster dec(3)  

outreg2 using ./tables/Table_2.tex, append ct("APEs") eqkeep(APEs) keep(ln_mean ln_gini) noobs ///
	nocons  noaster nonotes tex(frag) dec(3) 
	
restore
***********************************

*********** Table 3 & 4 ***********
	
**** now extract and boot regional-time period wise predictions of eyex

xtset ccode_cov ryear

* interesting time groups, or just decades?
cap drop ygroups
recode ryear (1980/1990=0 "80s") (1990/1995=1 "early 90s") ///
	(1995/2000=2 "late 90s") (2000/2005=3 "early 00s") ///
	(2005/2010=4 "late 00s"), gen(ygroups)
tab ygroups

* means for regional predictions  

* have to pass these to Mata so they can be exported via parallel
* which does currently not support exporting stata matrices

forv i = 0(1)4 {
	mean ln_mean ln_gini if ygroup==`i', over(region)
	mat B = e(b)
	mata: B = st_matrix("B")
	mata: My`i' = B[1,1..6] 
	mata: Mg`i' = B[1,7..12] 
}


cap program drop elas_boot
program elas_boot, eclass
	syntax [if] [in], var(varname numeric) [EYDX]
version 12.0
tempname bb v2h meanbar conv

	* control function: 1st step
	reg ln_mean ln_rcp ln_gini ib(last).Ti##c.ln_rcp_bar ib(last).Ti##c.ln_gini_bar ///
		y_1981-yb_2009 S_2-S_4 Sb_* if Ti>1
	predict `v2h', resid

	bys newid (ryear): egen `meanbar' = mean(ln_mean) if e(sample)
	
	* control function: 2nd step, set iter 100 to unjam too long bs iterations
	fhetprob h_200d  ln_mean ln_gini `v2h' ib(last).Ti##c.`meanbar' ib(last).Ti##c.ln_rcp_bar ib(last).Ti##c.ln_gini_bar ///
		y_1981-yb_2009  S_2-S_4 Sb_* if Ti>1, het(ib(last).Ti)  iter(100)

	scalar `conv' = e(converged)

	* predicting for each region and different year groups:
	* plug-in, predict & average, only then elasticities are identified
	if "`eydx'" != "" { 
		local type = "eydx"
		} 
		else {
		local type = "dydx"
		}
	forv i = 0(1)4 {
	
		*get the matrices out of mata
		mata: st_matrix("My`i'", My`i')
		mata: st_matrix("Mg`i'", Mg`i')
	
		#delimit ;
		margins, `type'(`var') 
			at(ln_mean=`=My`i'[1,1]' ln_gini=`=Mg`i'[1,1]') 
			at(ln_mean=`=My`i'[1,2]' ln_gini=`=Mg`i'[1,2]') 
			at(ln_mean=`=My`i'[1,3]' ln_gini=`=Mg`i'[1,3]') 
			at(ln_mean=`=My`i'[1,4]' ln_gini=`=Mg`i'[1,4]') 
			at(ln_mean=`=My`i'[1,5]' ln_gini=`=Mg`i'[1,5]') 
			at(ln_mean=`=My`i'[1,6]' ln_gini=`=Mg`i'[1,6]') nose;
		#delimit cr
		if (`i' == 0) {
			matrix `bb' = r(b)
		} 
		else {
			matrix `bb' = `bb',r(b)
		}
	}
	#delimit ;
	mat coln `bb' = 
		80-90:EAP 80-90:ECA 80-90:LAC 80-90:MNA 80-90:SAS 80-90:SSA 
		90-95:EAP 90-95:ECA 90-95:LAC 90-95:MNA 90-95:SAS 90-95:SSA 
		95-00:EAP 95-00:ECA 95-00:LAC 95-00:MNA 95-00:SAS 95-00:SSA 
		00-05:EAP 00-05:ECA 00-05:LAC 00-05:MNA 00-05:SAS 00-05:SSA 
		05-10:EAP 05-10:ECA 05-10:LAC 05-10:MNA 05-10:SAS 05-10:SSA;
	#delimit cr	
	ereturn post `bb'
	ereturn scalar converged =`conv'
 
end

* must be reset in each boot iteration
xtset, clear
drop if Ti==1 

* Table 3 upper panel
parallel bs, exp(_b) mata reps(`=$bsreps') ///
 seeds(102030 203010 302010 201030 405060 506040 605040 504060 708090 809070) ///
 cluster(ccode_cov) ///
 idcluster(newid) nowarn reject(e(converged)==0): elas_boot, var(ln_mean) eydx

outreg2 using ./tables/Table_3_panel_a.tex, wide replace  noaster noobs nonotes dec(3)  tex(frag) 

* Table 3 lower panel
parallel bs, exp(_b) mata reps(`=$bsreps') ///
 seeds(010203 020301 030201 020103 040506 050604 060504 050406 070809 080907) ///
 cluster(ccode_cov) ///
 idcluster(newid) nowarn reject(e(converged)==0): elas_boot, var(ln_mean) 
 
outreg2 using ./tables/Table_3_panel_b.tex, wide replace  noaster noobs nonotes dec(3)  tex(frag) 

* Table 4 upper panel
parallel bs, exp(_b) mata reps(`=$bsreps') ///
 seeds(210203 220301 230201 220103 240506 250604 260504 250406 270809 280907) ///
 cluster(ccode_cov) ///
 idcluster(newid) nowarn reject(e(converged)==0): elas_boot, var(ln_gini) eydx
 
outreg2 using ./tables/Table_4_panel_a.tex, wide replace  noaster noobs nonotes dec(3)  tex(frag) 

* Table 4 lower panel
parallel bs, exp(_b) mata reps(`=$bsreps') ///
 seeds(410203 420301 430201 420103 440506 450604 460504 450406 470809 480907) ///
 cluster(ccode_cov) ///
 idcluster(newid) nowarn reject(e(converged)==0): elas_boot, var(ln_gini) 
 
outreg2 using ./tables/Table_4_panel_b.tex, wide replace  noaster noobs nonotes dec(3)  tex(frag) 

***********************************

****** For Figures made in R ******

* for partials graph, Figure B-1 to be made in R

preserve
	cap drop *_tilde
	*replace ln_rcp = ln(rcp/12)

	reg ln_mean ln_gini ib(last).Ti##c.ln_rcp_bar ib(last).Ti##c.ln_gini_bar ///
		y_1981-yb_2009 S_2-S_4 Sb_* if Ti>1, vce(cluster ccode_cov)
	predict y_tilde if e(sample), resid

	reg ln_rcp ln_gini ib(last).Ti##c.ln_rcp_bar ib(last).Ti##c.ln_gini_bar ///
		y_1981-yb_2009 S_2-S_4 Sb_* if Ti>1, vce(cluster ccode_cov)
	predict x_tilde if e(sample), resid

	reg y_tilde x_tilde, vce(cluster ccode_cov)
	keep ccode* year y_tilde x_tilde
	saveold ./data/partials.dta, replace
restore 

saveold ./data/4Rgraphs_ybar, version(12) replace

* solve for z/yt, data for Figure 2 to be made in R
preserve
	mat A = J(8,26,.)
	mat B = A
	local k = 1
	forv y = -2(.2)3.1 {
		local i = 1
		forv x = 0.25(0.1)0.95 {
			qui estimates restore varfhet
			qui replace ln_mean = ln(60.83*(2^`y'))
			qui replace ln_gini = ln(`x')
			qui margins, eydx(ln_mean ln_gini) nose post
			mat C = e(b)
			mat A[`i',`k'] = C[1,1]
			mat B[`i',`k'] = C[1,2]
			local ++i
		}
		local ++k
	}

	mat li A
	mat li B

	clear
	svmat A
	saveold ./data/novarpredict_eY_ybar, version(12) replace
	
	clear
	svmat B
	saveold ./data/novarpredict_eG_ybar, version(12) replace
restore

* solve for z/yt, data for Figure 2 to be made in R, semi-elasticities
preserve
	mat A = J(8,26,.)
	mat B = A
	local k = 1
	forv y = -2(.2)3.1 {
		local i = 1
		forv x = 0.25(0.1)0.95 {
			qui estimates restore varfhet
			qui replace ln_mean = ln(60.83*(2^`y'))
			qui replace ln_gini = ln(`x')
			qui margins, dydx(ln_mean ln_gini) nose post
			mat C = e(b)
			mat A[`i',`k'] = C[1,1]
			mat B[`i',`k'] = C[1,2]
			local ++i
		}
		local ++k
	}

	mat li A
	mat li B

	clear
	svmat A
	saveold ./data/novarpredict_dY_ybar, version(12) replace
	
	clear
	svmat B
	saveold ./data/novarpredict_dG_ybar, version(12) replace
restore

***********************************
*% EOF
